/*
 * Note: this file originally auto-generated by mib2c using
 *       version : 12077 $ of $ 
 *
 * $Id:$
 */
#ifndef DOT3STATSTABLE_DATA_SET_H
#define DOT3STATSTABLE_DATA_SET_H

#ifdef __cplusplus
extern          "C" {
#endif

    /*
     *********************************************************************
     * SET function declarations
     */

    /*
     *********************************************************************
     * SET Table declarations
     */


#ifdef __cplusplus
}
#endif
#endif                          /* DOT3STATSTABLE_DATA_SET_H */
