
  /*
   * This file was generated by mib2c and is intended for use as a mib module
   * for the ucd-snmp snmpd agent. Edited by Michael Baer
   * 
   * Portions of this file are copyrighted by:
   * Copyright (c) 2016 VMware, Inc. All rights reserved.
   * Use is subject to license terms specified in the COPYING file
   * distributed with the Net-SNMP package.
   */
 
  #ifndef _MIBGROUP_SNMPTARGETPARAMSENTRY_H
  #define _MIBGROUP_SNMPTARGETPARAMSENTRY_H
 
#include "snmpTargetParamsEntry_data.h"

  /*
   * function definitions 
   */
 
       void            init_snmpTargetParamsEntry(void);
       void            shutdown_snmpTargetParamsEntry(void);
       int             store_snmpTargetParamsEntry(int majorID, int minorID,
                                                   void *serverarg,
                                                   void *clientarg);
       extern FindVarMethod var_snmpTargetParamsEntry;
 
       void            snmpd_parse_config_targetParams(const char *, char *);
 
#ifndef NETSNMP_NO_WRITE_SUPPORT
     WriteMethod     write_snmpTargetParamsMPModel;
     WriteMethod     write_snmpTargetParamsSecModel;
     WriteMethod     write_snmpTargetParamsSecName;
     WriteMethod     write_snmpTargetParamsSecLevel;
     WriteMethod     write_snmpTargetParamsStorageType;
     WriteMethod     write_snmpTargetParamsRowStatus;
#endif /* !NETSNMP_NO_WRITE_SUPPORT */


#endif                          /* _MIBGROUP_SNMPTARGETPARAMSENTRY_H */
