/*
 * Note: this file originally auto-generated by mib2c using
 *  $
 */
#ifndef SNMPTLSTMPARAMSTABLE_H
#define SNMPTLSTMPARAMSTABLE_H

#define SNMP_TLS_TM_PARAMS_COUNT   SNMP_TLS_TM_BASE, 2, 2, 1, 4
#define SNMP_TLS_TM_PARAMS_CHANGED SNMP_TLS_TM_BASE, 2, 2, 1, 5
#define SNMP_TLS_TM_PARAMS_TABLE   SNMP_TLS_TM_BASE, 2, 2, 1, 6

/* function declarations */
void init_snmpTlstmParamsTable(void);
void initialize_table_snmpTlstmParamsTable(void);
#define SNMPTLSTMPARAMSTABLE_TIMEOUT  60

/* index definitions for table snmpTlstmParamsTable */
#define SNMPTARGETPARAMSNAME_MAX_SIZE  32

/* column number definitions for table snmpTlstmParamsTable */
#define COLUMN_SNMPTLSTMPARAMSCLIENTFINGERPRINT    1
#define COLUMN_SNMPTLSTMPARAMSSTORAGETYPE    2
#define COLUMN_SNMPTLSTMPARAMSROWSTATUS    3
    
#define SNMPTLSTMPARAMSCLIENTFINGERPRINT_MAX_SIZE    255
    
#define SNMPTLSTMPARAMSTABLE_MIN_COLUMN COLUMN_SNMPTLSTMPARAMSCLIENTFINGERPRINT
#define SNMPTLSTMPARAMSTABLE_MAX_COLUMN COLUMN_SNMPTLSTMPARAMSROWSTATUS

#endif /* SNMPTLSTMPARAMSTABLE_H */
