/*
 * Note: this file originally auto-generated by mib2c using
 *       version : 15899 $ of $
 *
 * $Id:ipv6ScopeZoneIndexTable_interface.h 14170 2007-04-29 00:12:32Z varun_c$
 */
/** @ingroup interface: Routines to interface to Net-SNMP
 *
 * \warning This code should not be modified, called directly,
 *          or used to interpret functionality. It is subject to
 *          change at any time.
 * 
 * @{
 */
/*
 * *********************************************************************
 * *********************************************************************
 * *********************************************************************
 * ***                                                               ***
 * ***  NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE  ***
 * ***                                                               ***
 * ***                                                               ***
 * ***       THIS FILE DOES NOT CONTAIN ANY USER EDITABLE CODE.      ***
 * ***                                                               ***
 * ***                                                               ***
 * ***       THE GENERATED CODE IS INTERNAL IMPLEMENTATION, AND      ***
 * ***                                                               ***
 * ***                                                               ***
 * ***    IS SUBJECT TO CHANGE WITHOUT WARNING IN FUTURE RELEASES.   ***
 * ***                                                               ***
 * ***                                                               ***
 * *********************************************************************
 * *********************************************************************
 * *********************************************************************
 */
#ifndef IPV6SCOPEZONEINDEXTABLE_INTERFACE_H
#define IPV6SCOPEZONEINDEXTABLE_INTERFACE_H

#ifdef __cplusplus
extern          "C" {
#endif


#include "ipv6ScopeZoneIndexTable.h"


    /*
     ********************************************************************
     * Table declarations
     */

    /*
     * PUBLIC interface initialization routine 
     */
    void           
        _ipv6ScopeZoneIndexTable_initialize_interface
        (ipv6ScopeZoneIndexTable_registration * user_ctx, u_long flags);
    void           
        _ipv6ScopeZoneIndexTable_shutdown_interface
        (ipv6ScopeZoneIndexTable_registration * user_ctx);

    ipv6ScopeZoneIndexTable_registration
        *ipv6ScopeZoneIndexTable_registration_get(void);

    ipv6ScopeZoneIndexTable_registration
        *ipv6ScopeZoneIndexTable_registration_set
        (ipv6ScopeZoneIndexTable_registration * newreg);

    netsnmp_container *ipv6ScopeZoneIndexTable_container_get(void);
    int             ipv6ScopeZoneIndexTable_container_size(void);

    ipv6ScopeZoneIndexTable_rowreq_ctx
        *ipv6ScopeZoneIndexTable_allocate_rowreq_ctx(ipv6ScopeZoneIndexTable_data *, void *);
    void           
        ipv6ScopeZoneIndexTable_release_rowreq_ctx
        (ipv6ScopeZoneIndexTable_rowreq_ctx * rowreq_ctx);

    int             ipv6ScopeZoneIndexTable_index_to_oid(netsnmp_index *
                                                         oid_idx,
                                                         ipv6ScopeZoneIndexTable_mib_index
                                                         * mib_idx);
    int             ipv6ScopeZoneIndexTable_index_from_oid(netsnmp_index *
                                                           oid_idx,
                                                           ipv6ScopeZoneIndexTable_mib_index
                                                           * mib_idx);

    /*
     * access to certain internals. use with caution!
     */
    void           
        ipv6ScopeZoneIndexTable_valid_columns_set(netsnmp_column_info *vc);


#ifdef __cplusplus
}
#endif
#endif                          /* IPV6SCOPEZONEINDEXTABLE_INTERFACE_H */
/** @} */
