/*
 * Note: this file originally auto-generated by mib2c using
 *  : generic-table-enums.m2c,v 1.4 2005/07/15 22:41:16 rstory Exp $
 *
 * $Id$
 */
#ifndef USMDHUSERKEYTABLE_ENUMS_H
#define USMDHUSERKEYTABLE_ENUMS_H

#ifdef __cplusplus
extern          "C" {
#endif

    /*
     * NOTES on enums
     * ==============
     *
     * Value Mapping
     * -------------
     * If the values for your data type don't exactly match the
     * possible values defined by the mib, you should map them
     * below. For example, a boolean flag (1/0) is usually represented
     * as a TruthValue in a MIB, which maps to the values (1/2).
     *
     */
/*************************************************************************
 *************************************************************************
 *
 * enum definitions for table usmDHUserKeyTable
 *
 *************************************************************************
 *************************************************************************/



#ifdef __cplusplus
}
#endif
#endif                          /* USMDHUSERKEYTABLE_ENUMS_H */
