/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.table_data.conf,v 1.8 2005/07/19 14:52:03 dts12 Exp $
 */
#ifndef MTETRIGGERTABLE_H
#define MTETRIGGERTABLE_H

config_require(utilities/iquery)

/*
 * function declarations 
 */
void            init_mteTriggerTable(void);
void            shutdown_mteTriggerTable(void);
Netsnmp_Node_Handler mteTriggerTable_handler;

/*
 * column number definitions for table mteTriggerTable 
 */
#define COLUMN_MTEOWNER				 1
#define COLUMN_MTETRIGGERNAME			 2
#define COLUMN_MTETRIGGERCOMMENT		 3
#define COLUMN_MTETRIGGERTEST			 4
#define COLUMN_MTETRIGGERSAMPLETYPE		 5
#define COLUMN_MTETRIGGERVALUEID		 6
#define COLUMN_MTETRIGGERVALUEIDWILDCARD	 7
#define COLUMN_MTETRIGGERTARGETTAG		 8
#define COLUMN_MTETRIGGERCONTEXTNAME		 9
#define COLUMN_MTETRIGGERCONTEXTNAMEWILDCARD	10
#define COLUMN_MTETRIGGERFREQUENCY		11
#define COLUMN_MTETRIGGEROBJECTSOWNER		12
#define COLUMN_MTETRIGGEROBJECTS		13
#define COLUMN_MTETRIGGERENABLED		14
#define COLUMN_MTETRIGGERENTRYSTATUS		15
#endif                          /* MTETRIGGERTABLE_H */
