/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.table_data.conf,v 1.8 2005/07/19 14:52:03 dts12 Exp $
 */
#ifndef MTETRIGGEREXISTENCETABLE_H
#define MTETRIGGEREXISTENCETABLE_H

/*
 * function declarations 
 */
void            init_mteTriggerExistenceTable(void);
Netsnmp_Node_Handler mteTriggerExistenceTable_handler;

/*
 * column number definitions for table mteTriggerExistenceTable 
 */
#define COLUMN_MTETRIGGEREXISTENCETEST		1
#define COLUMN_MTETRIGGEREXISTENCESTARTUP	2
#define COLUMN_MTETRIGGEREXISTENCEOBJECTSOWNER	3
#define COLUMN_MTETRIGGEREXISTENCEOBJECTS	4
#define COLUMN_MTETRIGGEREXISTENCEEVENTOWNER	5
#define COLUMN_MTETRIGGEREXISTENCEEVENT		6
#endif                          /* MTETRIGGEREXISTENCETABLE_H */
