/*
 * Note: this file originally auto-generated by mib2c 
 */
#ifndef NSTRANSACTIONTABLE_H
#define NSTRANSACTIONTABLE_H

/*
 * function declarations 
 */
void            init_nsTransactionTable(void);
void            initialize_table_nsTransactionTable(void);
Netsnmp_Node_Handler nsTransactionTable_handler;
Netsnmp_First_Data_Point nsTransactionTable_get_first_data_point;
Netsnmp_Next_Data_Point nsTransactionTable_get_next_data_point;

/*
 * column number definitions for table nsTransactionTable 
 */
#define COLUMN_NSTRANSACTIONID		1
#define COLUMN_NSTRANSACTIONMODE		2
#endif                          /* NSTRANSACTIONTABLE_H */
