/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.table_data.conf,v 1.11 2005/09/16 11:48:12 dts12 Exp $
 */
#ifndef EXPEXPRESSIONERRORTABLE_H
#define EXPEXPRESSIONERRORTABLE_H

/*
 * function declarations 
 */
void            init_expErrorTable(void);
Netsnmp_Node_Handler expErrorTable_handler;

/*
 * column number definitions for table expExpressionErrorTable 
 */
#define COLUMN_EXPERRORTIME			1
#define COLUMN_EXPERRORINDEX			2
#define COLUMN_EXPERRORCODE			3
#define COLUMN_EXPERRORINSTANCE			4
#endif                          /* EXPEXPRESSIONERRORTABLE_H */
