/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.table_data.conf,v 1.8 2005/07/19 14:52:03 dts12 Exp $
 */
#ifndef MTETRIGGERDELTATABLE_H
#define MTETRIGGERDELTATABLE_H

/*
 * function declarations 
 */
void            init_mteTriggerDeltaTable(void);
Netsnmp_Node_Handler mteTriggerDeltaTable_handler;

/*
 * column number definitions for table mteTriggerDeltaTable 
 */
#define COLUMN_MTETRIGGERDELTADISCONTINUITYID		1
#define COLUMN_MTETRIGGERDELTADISCONTINUITYIDWILDCARD	2
#define COLUMN_MTETRIGGERDELTADISCONTINUITYIDTYPE	3
#endif                          /* MTETRIGGERDELTATABLE_H */
