#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-features.h>

netsnmp_feature_require(enable_stderrlog);
netsnmp_feature_require(memory_malloc);
netsnmp_feature_require(memory_free);

