/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.column_defines.conf,v 5.1 2002/05/08 05:42:47 hardaker Exp $
 */
#ifndef IPCIDRROUTETABLE_COLUMNS_H
#define IPCIDRROUTETABLE_COLUMNS_H

/*
 * column number definitions for table ipCidrRouteTable 
 */
#define COLUMN_IPCIDRROUTEDEST		1
#define COLUMN_IPCIDRROUTEMASK		2
#define COLUMN_IPCIDRROUTETOS		3
#define COLUMN_IPCIDRROUTENEXTHOP		4
#define COLUMN_IPCIDRROUTEIFINDEX		5
#define COLUMN_IPCIDRROUTETYPE		6
#define COLUMN_IPCIDRROUTEPROTO		7
#define COLUMN_IPCIDRROUTEAGE		8
#define COLUMN_IPCIDRROUTEINFO		9
#define COLUMN_IPCIDRROUTENEXTHOPAS		10
#define COLUMN_IPCIDRROUTEMETRIC1		11
#define COLUMN_IPCIDRROUTEMETRIC2		12
#define COLUMN_IPCIDRROUTEMETRIC3		13
#define COLUMN_IPCIDRROUTEMETRIC4		14
#define COLUMN_IPCIDRROUTEMETRIC5		15
#define COLUMN_IPCIDRROUTESTATUS		16
#endif                          /* IPCIDRROUTETABLE_COLUMNS_H */
