/*
 * Note: this file originally auto-generated by mib2c using
 *  : generic-table-oids.m2c 12855 2005-09-27 15:56:08Z rstory $
 *
 * $Id:$
 */
#ifndef IPDEFAULTROUTERTABLE_OIDS_H
#define IPDEFAULTROUTERTABLE_OIDS_H

#ifdef __cplusplus
extern          "C" {
#endif


    /*
     * column number definitions for table ipDefaultRouterTable 
     */
#define IPDEFAULTROUTERTABLE_OID              1,3,6,1,2,1,4,37

#define COLUMN_IPDEFAULTROUTERADDRESSTYPE         1

#define COLUMN_IPDEFAULTROUTERADDRESS         2

#define COLUMN_IPDEFAULTROUTERIFINDEX         3

#define COLUMN_IPDEFAULTROUTERLIFETIME         4

#define COLUMN_IPDEFAULTROUTERPREFERENCE         5


#define IPDEFAULTROUTERTABLE_MIN_COL   COLUMN_IPDEFAULTROUTERLIFETIME
#define IPDEFAULTROUTERTABLE_MAX_COL   COLUMN_IPDEFAULTROUTERPREFERENCE



#ifdef __cplusplus
}
#endif
#endif                          /* IPDEFAULTROUTERTABLE_OIDS_H */
