/*
 *  memory quantity mib groups
 *
 */
#ifndef _MIBGROUP_MEMORY_HPUX_H
#define _MIBGROUP_MEMORY_HPUX_H

#include "mibdefs.h"

void            init_memory_hpux(void);

#endif                          /* _MIBGROUP_MEMORY_HPUX_H */
