/*
 * This file was generated by mib2c and is intended for use as a mib module
 * for the ucd-snmp snmpd agent. 
 *
 * Portions of this file are copyrighted by:
 * Copyright (c) 2016 VMware, Inc. All rights reserved.
 * Use is subject to license terms specified in the COPYING file
 * distributed with the Net-SNMP package.
 */


#ifndef _MIBGROUP_SNMPNOTIFYFILTERPROFILETABLE_H
#define _MIBGROUP_SNMPNOTIFYFILTERPROFILETABLE_H


/*
 * we may use header_complex from the header_complex module 
 */
config_require(notification/snmpNotifyFilterProfileTable_data)

#include "snmpNotifyFilterProfileTable_data.h"


/*
 * function prototypes 
 */
     void            init_snmpNotifyFilterProfileTable(void);
     void            shutdown_snmpNotifyFilterProfileTable(void);
     FindVarMethod   var_snmpNotifyFilterProfileTable;
     void            parse_snmpNotifyFilterProfileTable(const char *,
                                                        char *);
     SNMPCallback    store_snmpNotifyFilterProfileTable;

#ifndef NETSNMP_NO_WRITE_SUPPORT 
     WriteMethod     write_snmpNotifyFilterProfileName;
     WriteMethod     write_snmpNotifyFilterProfileStorType;
     WriteMethod     write_snmpNotifyFilterProfileRowStatus;
#endif /* !NETSNMP_NO_WRITE_SUPPORT */

#endif                          /* _MIBGROUP_SNMPNOTIFYFILTERPROFILETABLE_H */
