/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.table_data.conf,v 1.8 2005/07/19 14:52:03 dts12 Exp $
 */
#ifndef MTEOBJECTSTABLE_H
#define MTEOBJECTSTABLE_H

/*
 * function declarations 
 */
void            init_mteObjectsTable(void);
void            shutdown_mteObjectsTable(void);
Netsnmp_Node_Handler mteObjectsTable_handler;

/*
 * column number definitions for table mteObjectsTable 
 */
#define COLUMN_MTEOBJECTSNAME		1
#define COLUMN_MTEOBJECTSINDEX		2
#define COLUMN_MTEOBJECTSID		3
#define COLUMN_MTEOBJECTSIDWILDCARD	4
#define COLUMN_MTEOBJECTSENTRYSTATUS	5
#endif                          /* MTEOBJECTSTABLE_H */
