/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.table_data.conf,v 1.8 2005/07/19 14:52:03 dts12 Exp $
 */
#ifndef MTETRIGGERTHRESHOLDTABLE_H
#define MTETRIGGERTHRESHOLDTABLE_H

/*
 * function declarations 
 */
void            init_mteTriggerThresholdTable(void);
void            shutdown_mteTriggerThresholdTable(void);
Netsnmp_Node_Handler mteTriggerThresholdTable_handler;

/*
 * column number definitions for table mteTriggerThresholdTable 
 */
#define COLUMN_MTETRIGGERTHRESHOLDSTARTUP		  1
#define COLUMN_MTETRIGGERTHRESHOLDRISING		  2
#define COLUMN_MTETRIGGERTHRESHOLDFALLING		  3
#define COLUMN_MTETRIGGERTHRESHOLDDELTARISING		  4
#define COLUMN_MTETRIGGERTHRESHOLDDELTAFALLING		  5
#define COLUMN_MTETRIGGERTHRESHOLDOBJECTSOWNER		  6
#define COLUMN_MTETRIGGERTHRESHOLDOBJECTS		  7
#define COLUMN_MTETRIGGERTHRESHOLDRISINGEVENTOWNER	  8
#define COLUMN_MTETRIGGERTHRESHOLDRISINGEVENT		  9
#define COLUMN_MTETRIGGERTHRESHOLDFALLINGEVENTOWNER	 10
#define COLUMN_MTETRIGGERTHRESHOLDFALLINGEVENT		 11
#define COLUMN_MTETRIGGERTHRESHOLDDELTARISINGEVENTOWNER	 12
#define COLUMN_MTETRIGGERTHRESHOLDDELTARISINGEVENT	 13
#define COLUMN_MTETRIGGERTHRESHOLDDELTAFALLINGEVENTOWNER 14
#define COLUMN_MTETRIGGERTHRESHOLDDELTAFALLINGEVENT	 15

#endif                          /* MTETRIGGERTHRESHOLDTABLE_H */
