/* This file is automatically generated by configure.  Do not modify by hand. */
  if (should_init("usmConf")) init_usmConf();
  if (should_init("iquery")) init_iquery();
  if (should_init("vacm_conf")) init_vacm_conf();

