/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.column_defines.conf,v 5.1 2002/05/08 05:42:47 hardaker Exp $
 */
#ifndef NETSNMPHOSTSTABLE_COLUMNS_H
#define NETSNMPHOSTSTABLE_COLUMNS_H

/*
 * column number definitions for table netSnmpHostsTable 
 */
#define COLUMN_NETSNMPHOSTNAME		1
#define COLUMN_NETSNMPHOSTADDRESSTYPE		2
#define COLUMN_NETSNMPHOSTADDRESS		3
#define COLUMN_NETSNMPHOSTSTORAGE		4
#define COLUMN_NETSNMPHOSTROWSTATUS		5
#endif                          /* NETSNMPHOSTSTABLE_COLUMNS_H */
