
/*
 * This file was generated by mib2c and is intended for use as a mib module
 * for the ucd-snmp snmpd agent.  Edited by Michael Baer
 * 
 * Portions of this file are copyrighted by:
 * Copyright (c) 2016 VMware, Inc. All rights reserved.
 * Use is subject to license terms specified in the COPYING file
 * distributed with the Net-SNMP package.
 */

#ifndef _MIBGROUP_SNMPTARGETADDRENTRY_H
#define _MIBGROUP_SNMPTARGETADDRENTRY_H

/*
 * we use header_generic from the util_funcs module
 */

config_require(util_funcs/header_generic)

#include "snmpTargetAddrEntry_data.h"

    /*
     * add the SNMPv2-TM mib into the default list of mibs to load, since
     * it contains the Domain definitions (EG, netsnmpUDPDomain) 
     */
config_add_mib(SNMPv2-TM)


/*
 * function definitions 
 */

     void            init_snmpTargetAddrEntry(void);
     void            shutdown_snmpTargetAddrEntry(void);
     FindVarMethod   var_snmpTargetAddrEntry;

     void            snmpd_parse_config_targetAddr(const char *, char *);

#ifndef NETSNMP_NO_WRITE_SUPPORT
     WriteMethod     write_snmpTargetAddrTDomain;
     WriteMethod     write_snmpTargetAddrTAddress;
     WriteMethod     write_snmpTargetAddrTimeout;
     WriteMethod     write_snmpTargetAddrRetryCount;
     WriteMethod     write_snmpTargetAddrTagList;
     WriteMethod     write_snmpTargetAddrParams;
     WriteMethod     write_snmpTargetAddrStorageType;
     WriteMethod     write_snmpTargetAddrRowStatus;

     WriteMethod     write_targetSpinLock;
#endif /* !NETSNMP_NO_WRITE_SUPPORT */
     FindVarMethod   var_targetSpinLock;

#endif                          /* _MIBGROUP_SNMPTARGETADDRENTRY_H */
