/*
 * This file was generated by mib2c and is intended for use as a mib module
 * for the ucd-snmp snmpd agent. 
 *
 * Portions of this file are copyrighted by:
 * Copyright (c) 2016 VMware, Inc. All rights reserved.
 * Use is subject to license terms specified in the COPYING file
 * distributed with the Net-SNMP package.
 */


#ifndef _MIBGROUP_SNMPNOTIFYTABLE_H
#define _MIBGROUP_SNMPNOTIFYTABLE_H


/*
 * we may use header_complex from the header_complex module 
 */


config_require(notification/snmpNotifyTable_data)
config_require(target)
config_add_mib(SNMP-NOTIFICATION-MIB)

#include "snmpNotifyTable_data.h"


/*
 * function prototypes 
 */
     void            init_snmpNotifyTable(void);
     void            shutdown_snmpNotifyTable(void);
     FindVarMethod   var_snmpNotifyTable;
     void            parse_snmpNotifyTable(const char *, char *);

#ifndef NETSNMP_NO_WRITE_SUPPORT
     WriteMethod     write_snmpNotifyTag;
     WriteMethod     write_snmpNotifyType;
     WriteMethod     write_snmpNotifyStorageType;
     WriteMethod     write_snmpNotifyRowStatus;
#endif /* !NETSNMP_NO_WRITE_SUPPORT */



#endif                          /* _MIBGROUP_SNMPNOTIFYTABLE_H */
