
config_require(notification/snmpNotifyTable_data)
config_require(snmp-notification-mib/snmpNotifyFilterTable/snmpNotifyFilterTable_data_storage)
config_require(notification/snmpNotifyFilterProfileTable_data)
