/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.scalar.conf,v 1.7 2003/04/08 14:57:04 dts12 Exp $
 */
#ifndef USMDHPARAMETERS_H
#define USMDHPARAMETERS_H

#include <openssl/dh.h>

/*
 * function declarations 
 */
DH *            get_dh_params(void);
void            init_usmDHParameters(void);
Netsnmp_Node_Handler handle_usmDHParameters;

#endif                          /* USMDHPARAMETERS_H */
