/*
 * Note: this file originally auto-generated by mib2c using
 *        $
 */
#ifndef SNMPTSMCONFIGURATIONUSEPREFIX_H
#define SNMPTSMCONFIGURATIONUSEPREFIX_H

/*
 * function declarations 
 */
void            init_snmpTsmConfigurationUsePrefix(void);
Netsnmp_Node_Handler handle_snmpTsmConfigurationUsePrefix;

#endif                          /* SNMPTSMCONFIGURATIONUSEPREFIX_H */
