#define ifDescr ifentry->descr
#define ifType ifentry->type
#define ifMtu ifentry->mtu
#define ifSpeed ifentry->speed
#define ifPhysAddress ifentry->paddr
#define ifPhysAddress_len ifentry->paddr_len    /* # of char elements, not bytes */
#define ifAdminStatus ifentry->admin_status
#define ifOperStatus ifentry->oper_status
#define ifLastChange ifentry->lastchange
#define ifInOctets ifentry->stats.ibytes.low
#define ifInUcastPkts ifentry->stats.iucast.low
#define ifInNUcastPkts ifentry->stats.inucast
#define ifInDiscards ifentry->stats.idiscards
#define ifInErrors ifentry->stats.ierrors
#define ifInUnknownProtos ifentry->stats.iunknown_protos
#define ifOutOctets ifentry->stats.obytes.low
#define ifOutUcastPkts ifentry->stats.oucast.low
#define ifOutNUcastPkts ifentry->stats.onucast
#define ifOutDiscards ifentry->stats.odiscards
#define ifOutErrors ifentry->stats.oerrors
#define ifOutQLen ifentry->stats.oqlen
#define ifName ifentry->name
#define ifInMulticastPkts ifentry->stats.imcast.low
#define ifInBroadcastPkts ifentry->stats.ibcast.low
#define ifOutMulticastPkts ifentry->stats.omcast.low
#define ifOutBroadcastPkts ifentry->stats.obcast.low
#define ifHCInOctets ifentry->stats.ibytes
#define ifHCInUcastPkts ifentry->stats.iucast
#define ifHCInMulticastPkts ifentry->stats.imcast
#define ifHCInBroadcastPkts ifentry->stats.ibcast
#define ifHCOutOctets ifentry->stats.obytes
#define ifHCOutUcastPkts ifentry->stats.oucast
#define ifHCOutMulticastPkts ifentry->stats.omcast
#define ifHCOutBroadcastPkts ifentry->stats.obcast
#define ifHighSpeed ifentry->speed_high
#define ifPromiscuousMode ifentry->promiscuous
#define ifConnectorPresent ifentry->connector_present
