/*
 * Note: this file originally auto-generated by mib2c using
 *  : generic-table-oids.m2c 12855 2005-09-27 15:56:08Z rstory $
 *
 * $Id:ipv6ScopeZoneIndexTable_oids.h 14170 2007-04-29 00:12:32Z varun_c$
 */
#ifndef IPV6SCOPEZONEINDEXTABLE_OIDS_H
#define IPV6SCOPEZONEINDEXTABLE_OIDS_H

#ifdef __cplusplus
extern          "C" {
#endif


    /*
     * column number definitions for table ipv6ScopeZoneIndexTable 
     */
#define IPV6SCOPEZONEINDEXTABLE_OID              1,3,6,1,2,1,4,36

#define COLUMN_IPV6SCOPEZONEINDEXIFINDEX         1

#define COLUMN_IPV6SCOPEZONEINDEXLINKLOCAL         2

#define COLUMN_IPV6SCOPEZONEINDEX3         3

#define COLUMN_IPV6SCOPEZONEINDEXADMINLOCAL         4

#define COLUMN_IPV6SCOPEZONEINDEXSITELOCAL         5

#define COLUMN_IPV6SCOPEZONEINDEX6         6

#define COLUMN_IPV6SCOPEZONEINDEX7         7

#define COLUMN_IPV6SCOPEZONEINDEXORGANIZATIONLOCAL         8

#define COLUMN_IPV6SCOPEZONEINDEX9         9

#define COLUMN_IPV6SCOPEZONEINDEXA         10

#define COLUMN_IPV6SCOPEZONEINDEXB         11

#define COLUMN_IPV6SCOPEZONEINDEXC         12

#define COLUMN_IPV6SCOPEZONEINDEXD         13


#define IPV6SCOPEZONEINDEXTABLE_MIN_COL   COLUMN_IPV6SCOPEZONEINDEXLINKLOCAL
#define IPV6SCOPEZONEINDEXTABLE_MAX_COL   COLUMN_IPV6SCOPEZONEINDEXD



#ifdef __cplusplus
}
#endif
#endif                          /* IPV6SCOPEZONEINDEXTABLE_OIDS_H */
