/*
 * Note: this file originally auto-generated by mib2c using
 *  : mib2c.create-dataset.conf,v 5.2 2002/07/17 14:41:53 dts12 Exp $
 */
#ifndef MTEEVENTNOTIFICATIONTABLE_H
#define MTEEVENTNOTIFICATIONTABLE_H

/*
 * function declarations 
 */
void            init_mteEventNotificationTable(void);
void            initialize_table_mteEventNotificationTable(void);
Netsnmp_Node_Handler mteEventNotificationTable_handler;

/*
 * column number definitions for table mteEventNotificationTable 
 */
#define COLUMN_MTEEVENTNOTIFICATION		1
#define COLUMN_MTEEVENTNOTIFICATIONOBJECTSOWNER		2
#define COLUMN_MTEEVENTNOTIFICATIONOBJECTS		3

extern netsnmp_table_data_set *mteEventNotif_table_set;
#endif                          /* MTEEVENTNOTIFICATIONTABLE_H */
