/* This file is automatically generated by configure.  Do not modify by hand. */

#ifndef MIB_MODULE_CONFIG_H
#define MIB_MODULE_CONFIG_H

/* Define if compiling with the mibII module files.  */
#define USING_MIBII_MODULE 1

/* Define if compiling with the ucd_snmp module files.  */
#define USING_UCD_SNMP_MODULE 1

/* Define if compiling with the snmpv3mibs module files.  */
#define USING_SNMPV3MIBS_MODULE 1
 
/* Define if compiling with the notification module files.  */
#define USING_NOTIFICATION_MODULE 1
 
/* Define if compiling with the disman/event module files.  */
#define USING_DISMAN_EVENT_MODULE 1
 
/* Define if compiling with the notification-log-mib module files. */
#ifdef HAVE_WIN32_PLATFORM_SDK
#define USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE 1
#endif

/* Define if compiling with the target module files.  */
#define USING_TARGET_MODULE 1
 
/* Define if compiling with the examples/ucdDemoPublic module files.  */
#define USING_EXAMPLES_UCDDEMOPUBLIC_MODULE 1
 
/* Define if compiling with the examples/example module files.  */
#define USING_EXAMPLES_EXAMPLE_MODULE 1
 
/* Define if compiling with the agent_mibs module files.  */
#define USING_AGENT_MIBS_MODULE 1
 
/* Define if compiling with the agent_mibs module files.  */
#define USING_AGENT_MIBS_MODULE 1
 
/* Define if compiling with the utilities module files.  */
#define USING_UTILITIES_MODULE 1

/* Define if compiling with the mibII/ipv6 module files.  */
/* #undef USING_MIBII_IPV6_MODULE */
 
/* Define if compiling with the mibII/system_mib module files.  */
#define USING_MIBII_SYSTEM_MIB_MODULE 1
 
/* Define if compiling with the mibII/sysORTable module files.  */
#define USING_MIBII_SYSORTABLE_MODULE 1
 
/* Define if compiling with the mibII/at module files.  */
#define USING_MIBII_AT_MODULE 1
 
#if 0
/* Define if compiling with the mibII/interfaces module files.  */
#define USING_MIBII_INTERFACES_MODULE 1
#else
/* Define if compiling with the if-mib module files.  */
#define USING_IF_MIB_MODULE 1

/* Define if compiling with the if-mib/ifTable module files.  */
#define USING_IF_MIB_IFTABLE_MODULE 1

/* Define if compiling with the if-mib/ifXTable module files.  */
#define USING_IF_MIB_IFXTABLE_MODULE 1

/* Define if compiling with the if-mib/ifTable/ifTable module files.  */
#define USING_IF_MIB_IFTABLE_IFTABLE_MODULE 1

/* Define if compiling with the if-mib/ifXTable/ifXTable module files.  */
#define USING_IF_MIB_IFXTABLE_IFXTABLE_MODULE 1

/* Define if compiling with the if-mib/data_access/interface module files.  */
#define USING_IF_MIB_DATA_ACCESS_INTERFACE_MODULE 1

/* Define if compiling with the if-mib/ifTable/ifTable_interface module files.  */
#define USING_IF_MIB_IFTABLE_IFTABLE_INTERFACE_MODULE 1

/* Define if compiling with the if-mib/ifTable/ifTable_data_access module files.  */
#define USING_IF_MIB_IFTABLE_IFTABLE_DATA_ACCESS_MODULE 1

/* Define if compiling with the if-mib/ifXTable/ifXTable_interface module files.  */
#define USING_IF_MIB_IFXTABLE_IFXTABLE_INTERFACE_MODULE 1

/* Define if compiling with the if-mib/ifXTable/ifXTable_data_access module files.  */
#define USING_IF_MIB_IFXTABLE_IFXTABLE_DATA_ACCESS_MODULE 1

/* Define if compiling with the if-mib/data_access/interface_iphlpapi module files.  */
#define USING_IF_MIB_DATA_ACCESS_INTERFACE_IPHLPAPI_MODULE 1
#endif
 
/* Define if compiling with the mibII/snmp_mib module files.  */
#define USING_MIBII_SNMP_MIB_MODULE 1
 
/* Define if compiling with the mibII/tcp module files.  */
#define USING_MIBII_TCP_MODULE 1
 
/* Define if compiling with the mibII/icmp module files.  */
#define USING_MIBII_ICMP_MODULE 1
 
/* Define if compiling with the mibII/ip module files.  */
#define USING_MIBII_IP_MODULE 1
 
/* Define if compiling with the mibII/udp module files.  */
#define USING_MIBII_UDP_MODULE 1
 
/* Define if compiling with the mibII/vacm_vars module files.  */
#define USING_MIBII_VACM_VARS_MODULE 1
 
/* Define if compiling with the mibII/setSerialNo module files.  */
#define USING_MIBII_SETSERIALNO_MODULE 1


#ifdef USING_UCD_SNMP_MODULE

/* Define if compiling with the ucd-snmp/memory module files.  */
#define USING_UCD_SNMP_MEMORY_MODULE 1
 
/* Define if compiling with the ucd-snmp/vmstat module files.  */
#define USING_UCD_SNMP_VMSTAT_MODULE 1
 
/* Define if compiling with the ucd-snmp/proc module files.  */
#define USING_UCD_SNMP_PROC_MODULE 1
 
/* Define if compiling with the ucd-snmp/versioninfo module files.  */
#define USING_UCD_SNMP_VERSIONINFO_MODULE 1
 
/* Define if compiling with the ucd-snmp/pass module files.  */
#define USING_UCD_SNMP_PASS_MODULE 1
 
/* Define if compiling with the ucd-snmp/pass_persist module files.  */
#define USING_UCD_SNMP_PASS_PERSIST_MODULE 1
 
/* Define if compiling with the ucd-snmp/disk module files.  */
#define USING_UCD_SNMP_DISK_MODULE 1
 
/* Define if compiling with the ucd-snmp/loadave module files.  */
#define USING_UCD_SNMP_LOADAVE_MODULE 1
 
/* Define if compiling with the ucd-snmp/extensible module files.  */
#define USING_UCD_SNMP_EXTENSIBLE_MODULE 1
 
/* Define if compiling with the agent/extend module files.  */
#define USING_AGENT_EXTEND_MODULE 1
 
/* Define if compiling with the ucd-snmp/errormib module files.  */
#define USING_UCD_SNMP_ERRORMIB_MODULE 1
 
/* Define if compiling with the ucd-snmp/file module files.  */
#define USING_UCD_SNMP_FILE_MODULE 1
 
/* Define if compiling with the ucd-snmp/dlmod module files.  */
#define USING_UCD_SNMP_DLMOD_MODULE 1
 
/* Define if compiling with the ucd-snmp/proxy module files.  */
#define USING_UCD_SNMP_PROXY_MODULE 1

#endif /* USING_UCD_SNMP_MODULE */

 
#if HAVE_REGEX_H
/* Define if compiling with the ucd-snmp/logmatch module files.  */
#define USING_UCD_SNMP_LOGMATCH_MODULE 1
#endif
 
/* Define if compiling with the snmpv3/snmpEngine module files.  */
#define USING_SNMPV3_SNMPENGINE_MODULE 1
 
/* Define if compiling with the snmpv3/snmpMPDStats module files.  */
#define USING_SNMPV3_SNMPMPDSTATS_MODULE 1
 
/* Define if compiling with the snmpv3/usmStats module files.  */
#define USING_SNMPV3_USMSTATS_MODULE 1
 
/* Define if compiling with the snmpv3/usmUser module files.  */
#define USING_SNMPV3_USMUSER_MODULE 1

/* Define if compiling with the notification/snmpNotifyTable module files.  */
#define USING_NOTIFICATION_SNMPNOTIFYTABLE_MODULE 1
 
/* Define if compiling with the notification/snmpNotifyFilterTable module files.  */
#define USING_NOTIFICATION_SNMPNOTIFYFILTERTABLE_MODULE 1
 
/* Define if compiling with the notification/snmpNotifyFilterProfileTable module files.  */
#define USING_NOTIFICATION_SNMPNOTIFYFILTERPROFILETABLE_MODULE 1
 
/* Define if compiling with the target/snmpTargetAddrEntry module files.  */
#define USING_TARGET_SNMPTARGETADDRENTRY_MODULE 1
 
/* Define if compiling with the target/snmpTargetParamsEntry module files.  */
#define USING_TARGET_SNMPTARGETPARAMSENTRY_MODULE 1
 
/* Define if compiling with the target/target module files.  */
#define USING_TARGET_TARGET_MODULE 1
 
/* Define if compiling with the target/target_counters module files.  */
#define USING_TARGET_TARGET_COUNTERS_MODULE 1
 
/* Define if compiling with the agent/nsTransactionTable module files.  */
#define USING_AGENT_NSTRANSACTIONTABLE_MODULE 1
 
/* Define if compiling with the agent/nsModuleTable module files.  */
#define USING_AGENT_NSMODULETABLE_MODULE 1
 
/* Define if compiling with the agent/nsDebug module files.  */
#define USING_AGENT_NSDEBUG_MODULE 1
 
/* Define if compiling with the agent/nsCache module files.  */
#define USING_AGENT_NSCACHE_MODULE 1
 
/* Define if compiling with the agent/nsLogging module files.  */
#define USING_AGENT_NSLOGGING_MODULE 1
 
/* Define if compiling with the utilities/override module files.  */
#define USING_UTILITIES_OVERRIDE_MODULE 1

 
#ifdef USING_HOST_MODULE

/* Define if compiling with the host/hr_system module files.  */
#define USING_HOST_HR_SYSTEM_MODULE 1
 
/* Define if compiling with the host/hr_storage module files.  */
#define USING_HOST_HR_STORAGE_MODULE 1
 
/* Define if compiling with the host/hr_device module files.  */
#define USING_HOST_HR_DEVICE_MODULE 1
 
/* Define if compiling with the host/hr_other module files.  */
#define USING_HOST_HR_OTHER_MODULE 1
 
/* Define if compiling with the host/hr_proc module files.  */
#define USING_HOST_HR_PROC_MODULE 1
 
/* Define if compiling with the host/hr_network module files.  */
#define USING_HOST_HR_NETWORK_MODULE 1
 
/* Define if compiling with the host/hr_print module files.  */
#define USING_HOST_HR_PRINT_MODULE 1
 
/* Define if compiling with the host/hr_disk module files.  */
#define USING_HOST_HR_DISK_MODULE 1
 
/* Define if compiling with the host/hr_partition module files.  */
#define USING_HOST_HR_PARTITION_MODULE 1
 
/* Define if compiling with the host/hr_filesys module files.  */
#define USING_HOST_HR_FILESYS_MODULE 1
 
/* Define if compiling with the host/hr_swrun module files.  */
#define USING_HOST_HR_SWRUN_MODULE 1
 
/* Define if compiling with the host/hr_swinst module files.  */
#define USING_HOST_HR_SWINST_MODULE 1

#endif  /* USING_HOST_MODULE */
 

#ifdef USING_RMON_MODULE

/* Define if compiling with the Rmon/rows module files.  */
#define USING_RMON_ROWS_MODULE 1
 
/* Define if compiling with the Rmon/agutil module files.  */
#define USING_RMON_AGUTIL_MODULE 1
 
/* Define if compiling with the Rmon/statistics module files.  */
#define USING_RMON_STATISTICS_MODULE 1
 
/* Define if compiling with the Rmon/alarm module files.  */
#define USING_RMON_ALARM_MODULE 1
 
/* Define if compiling with the Rmon/history module files.  */
#define USING_RMON_HISTORY_MODULE 1
 
/* Define if compiling with the Rmon/event module files.  */
#define USING_RMON_EVENT_MODULE 1
 
#endif /* USING_RMON_MODULE */


#ifdef USING_DISMAN_EVENT_MODULE 
 
/* Define if compiling with the disman/event/mteScalars module files.  */
#define USING_DISMAN_EVENT_MTESCALARS_MODULE 1
 
/* Define if compiling with the disman/event/mteTrigger module files.  */
#define USING_DISMAN_EVENT_MTETRIGGER_MODULE 1
 
/* Define if compiling with the disman/event/mteTriggerTable module files.  */
#define USING_DISMAN_EVENT_MTETRIGGERTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteTriggerDeltaTable module files.  */
#define USING_DISMAN_EVENT_MTETRIGGERDELTATABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteTriggerExistenceTable module files.  */
#define USING_DISMAN_EVENT_MTETRIGGEREXISTENCETABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteTriggerBooleanTable module files.  */
#define USING_DISMAN_EVENT_MTETRIGGERBOOLEANTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteTriggerThresholdTable module files.  */
#define USING_DISMAN_EVENT_MTETRIGGERTHRESHOLDTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteTriggerConf module files.  */
#define USING_DISMAN_EVENT_MTETRIGGERCONF_MODULE 1
 
/* Define if compiling with the disman/event/mteEvent module files.  */
#define USING_DISMAN_EVENT_MTEEVENT_MODULE 1
 
/* Define if compiling with the disman/event/mteEventTable module files.  */
#define USING_DISMAN_EVENT_MTEEVENTTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteEventSetTable module files.  */
#define USING_DISMAN_EVENT_MTEEVENTSETTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteEventNotificationTable module files.  */
#define USING_DISMAN_EVENT_MTEEVENTNOTIFICATIONTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteEventConf module files.  */
#define USING_DISMAN_EVENT_MTEEVENTCONF_MODULE 1
 
/* Define if compiling with the disman/event/mteObjects module files.  */
#define USING_DISMAN_EVENT_MTEOBJECTS_MODULE 1
 
/* Define if compiling with the disman/event/mteObjectsTable module files.  */
#define USING_DISMAN_EVENT_MTEOBJECTSTABLE_MODULE 1
 
/* Define if compiling with the disman/event/mteObjectsConf module files.  */
#define USING_DISMAN_EVENT_MTEOBJECTSCONF_MODULE 1
 
#endif /* USING_DISMAN_EVENT_MODULE */

#ifdef USING_SMUX_MODULE
 
/* Define if compiling with the smux/smux module files.  */
#define USING_SMUX_SMUX_MODULE 1
 
#endif /* USING_SMUX_MODULE */

 
/* Define if compiling with the mibII/var_route module files.  */
#define USING_MIBII_VAR_ROUTE_MODULE 1
 
/* Define if compiling with the mibII/route_write module files.  */
#define USING_MIBII_ROUTE_WRITE_MODULE 1
 
/* Define if compiling with the util_funcs module files.  */
#define USING_UTIL_FUNCS_MODULE 1
 
/* Define if compiling with the mibII/tcpTable module files.  */
#define USING_MIBII_TCPTABLE_MODULE 1
 
/* Define if compiling with the mibII/ipAddr module files.  */
#define USING_MIBII_IPADDR_MODULE 1
 
/* Define if compiling with the mibII/udpTable module files.  */
#define USING_MIBII_UDPTABLE_MODULE 1
 
/* Define if compiling with the mibII/vacm_context module files.  */
#define USING_MIBII_VACM_CONTEXT_MODULE 1
 
/* Define if compiling with the header_complex module files.  */
#define USING_HEADER_COMPLEX_MODULE 1
 

#endif /* MIB_MODULE_CONFIG_H */
 
