/*
 * A simple helpful wrapper to include data access includes.
 */
#ifndef NETSNMP_DATA_ACCESS_INCLUDES
#define NETSNMP_DATA_ACCESS_INCLUDES

#include <net-snmp/data_access/arp.h>
#include <net-snmp/data_access/interface.h>
#include <net-snmp/data_access/ipaddress.h>
#include <net-snmp/data_access/route.h>
#include <net-snmp/data_access/ipstats.h>
#include <net-snmp/data_access/systemstats.h>

#endif /* NETSNMP_DATA_ACCESS_INCLUDES */
